﻿using CallCenter.Caches;
using CallCenter.Calls;
using CallCenter.NewRock.Handlers;
using CallCenter.Notifications;
using CallCenter.Realtimes;
using CallCenter.Share.Dtos;
using CallCenter.Share.Enums;
using CallCenter.Users;
using MediatR;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using NewRock.Sdk.Extensions;
using System.Collections.Generic;
using System.Text;
using System.Text.Json;
using XF.Domain.Exceptions;

namespace CallCenter.Application.Handlers
{
    public class CdrNotificationHandler:INotificationHandler<CdrNotification>
    {
        private readonly ICallRecordRepository _callRecordRepository;
        private readonly ICallDetailRepository _callDetailRepository;
        private readonly ICallRepository _callRepository;
        private readonly IOptionsSnapshot<SendCallRecord> _sendCallRecordOptions;
        private readonly IHttpClientFactory _httpClientFactory;
        private readonly ILogger<CdrNotificationHandler> _logger;
        private readonly IRealtimeService _realtimeService;
        private readonly IUserCacheManager _userCacheManager;
        private readonly IWorkRepository _workRepository;
        private readonly ICallRecordFailRepository _callRecordFailRepository;


        public CdrNotificationHandler(ICallRecordRepository callRecordRepository, ICallDetailRepository callDetailRepository, ICallRepository callRepository, IOptionsSnapshot<SendCallRecord> sendCallRecordOptions, IHttpClientFactory httpClientFactory,ILogger<CdrNotificationHandler> logger,IRealtimeService realtimeService,IUserCacheManager userCacheManager,IWorkRepository workRepository,ICallRecordFailRepository callRecordFailRepository)
        {
            _callRecordRepository = callRecordRepository;
            _callDetailRepository = callDetailRepository;
            _callRepository = callRepository;
            _sendCallRecordOptions = sendCallRecordOptions;
            _httpClientFactory = httpClientFactory;
            _logger = logger;
            _realtimeService = realtimeService;
            _userCacheManager = userCacheManager;
            _workRepository = workRepository;
            _callRecordFailRepository = callRecordFailRepository;
        }


        public async Task Handle(CdrNotification notification, CancellationToken cancellationToken)
        {

            if (notification.Type == "LO")
            {
                //通知前端主叫
                //获取主叫工作信息
                var fromWork = _userCacheManager.GetWorkByTel(notification.CPN);
                if (fromWork != null)
                {
                    await _realtimeService.ByeAsync(fromWork.UserId, new ByeDto() { Id = notification.CPN }, cancellationToken);
                }
                //通知前端被叫
                var toWork = _userCacheManager.GetWorkByTel(notification.CDPN);
                if (toWork != null)
                {
                    await _realtimeService.ByeAsync(toWork.UserId, new ByeDto() { Id = notification.CDPN }, cancellationToken);
                }
            }
            else
            {
                var callDetail = await
               _callDetailRepository.GetAsync(x => x.OMCallId == notification.CallId && x.CreationTime >= DateTime.Now.AddHours(-2), true, x => x.CreationTime, cancellationToken);

                if (callDetail != null)
                {
                    var model = new CallRecord()
                    {
                        CallId = callDetail.CallId,
                        CdrId = notification.Id,
                        CDRCallId = notification.CallId,
                        TimeStart = notification.TimeStart,
                        Group = notification.Group,
                        Type = (ECDRType)Enum.Parse(typeof(ECDRType), notification.Type),
                        Route = (ECDRRoute)Enum.Parse(typeof(ECDRRoute), notification.Route),
                        CPN = notification.CPN,
                        CDPN = notification.CDPN,
                        TimeEnd = notification.TimeEnd,
                        Duration = notification.Duration,
                        TrunkNumber = notification.TrunkNumber,
                        Recording = notification.Recording,
                        RecCodec = notification.RecCodec,
                        IsDown = false
                    };
                    if (!string.IsNullOrEmpty(notification.VisitorId))
                        model.VisitorId = notification.VisitorId;

                    if (!string.IsNullOrEmpty(notification.OuterId))
                        model.OuterId = notification.OuterId;

                    await _callRecordRepository.AddAsync(model, cancellationToken);
                    var callModel = await _callRepository.GetAsync(x => x.Id == callDetail.CallId, cancellationToken);

                    bool ishave = false;
                    if (callModel.CallType == ECallType.ExtToOuter)
                    {
                        ishave = true;
                    }
                    else
                    {
                        ishave = await _callDetailRepository.AnyAsync(x => (x.EventName == "RING" || x.EventName == "INCOMING") && x.OMCallId == notification.CallId && x.FromNo == notification.CPN && x.ToNo == notification.CDPN);
                    }

                    if (ishave)
                    {
                        if (callModel != null)
                        {
                            #region 推送通话记录报告

                            callModel.Duration = double.Parse(model.Duration);
                            await _callRepository.UpdateAsync(callModel, cancellationToken);

                            var call = await _callRepository.GetExtAsync(callModel.Id, x => x.Includes(d => d.CallDetails));
                            //推送通话报告
                            OutCallDto callDto = new OutCallDto();
                            callDto.CallId = callDetail.CallId;
                            callDto.InfoType = EInfoType.Call;
                            callDto.Direction = callModel.CallDirection;
                            callDto.Cpn = callModel.FromNo ?? "";
                            callDto.Cdpn = callModel.ToNo ?? "";
                            if (callDto.Direction == ECallDirection.In)
                            {
                                callDto.Answered = call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWER")?.AnswerNo ?? call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.AnswerNo ?? "";
                                callDto.OnTime = call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWER")?.CreationTime ?? call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.CreationTime ?? null;
                                callDto.OnState = call.CallDetails?.Any(x => x.EventName == "ANSWER" || x.EventName == "ANSWERED") == true ? EOnState.On : EOnState.NoOn;
                            }
                            else
                            {
                                //callDto.Answered = call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.AnswerNo ?? "";
                                //callDto.OnTime = call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.CreationTime;
                                //callDto.OnState = call.CallDetails?.Any(x => x.EventName == "ANSWERED") == true ? EOnState.On : EOnState.NoOn;
                                callDto.Answered = call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWER")?.AnswerNo ?? call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.AnswerNo ?? "";
                                callDto.OnTime = call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWER")?.CreationTime ?? call.CallDetails?.FirstOrDefault(x => x.EventName == "ANSWERED")?.CreationTime ?? null;
                                callDto.OnState = call.CallDetails?.Any(x => x.EventName == "ANSWER" || x.EventName == "ANSWERED") == true ? EOnState.On : EOnState.NoOn;
                            }
                            callDto.BeginTime = call.CreationTime;
                            callDto.ByeTime = call.CallDetails?.FirstOrDefault(x => x.EventName == "BYE")?.CreationTime ?? DateTime.Now;
                            callDto.TalkTime = call.Duration;
                            callDto.SoundFileName = model.Recording;
                            if (string.IsNullOrEmpty(callDto.SoundFileName))
                            {
                                callDto.SoundFileName = call.CallDetails?.FirstOrDefault(x => x.EventName == "BYE" && !string.IsNullOrEmpty(x.Recording))?.Recording;
                            }
                            if (string.IsNullOrEmpty(callDto.SoundFileName))
                            {
                                callDto.SoundFileName = (await _callRecordRepository.GetAsync(x => x.CallId == callModel.Id && !string.IsNullOrEmpty(x.Recording)))?.Recording;
                            }
                            callDto.EvaluateResult = call.CallDetails?.FirstOrDefault(x => x.EventName == "EVALUATE")?.Remark;
                            callDto.EndBy = call.EndBy;
                            callDto.InIvrTime = call.InIvrTime;
                            callDto.OutIvrTime = call.OutIvrTime;
                            callDto.InGroupTime = call.InGroupTime;
                            callDto.OutGroupTime = call.OutGroupTime;
                            callDto.InSeaTime = call.InSeaTime;
                            callDto.ConnSeaTime = call.ConnSeaTime;

                            var list = new List<OutCallDto>();
                            list.Add(callDto);

                            string respContentString = "";
                            try
                            {
                                using var client = _httpClientFactory.CreateClient();
                                client.DefaultRequestHeaders.ConnectionClose = true;
                                var requestContent = JsonSerializer.Serialize(list);
                                _logger.LogInformation(requestContent);
                                var content = new StringContent(requestContent, Encoding.UTF8, "application/json");
                                var responseMessage = await client.PostAsync(_sendCallRecordOptions.Value.FwUrl, content, cancellationToken);
                                var respContent = responseMessage.Content;
                                respContentString = await respContent.ReadAsStringAsync(cancellationToken);
                                var result = JsonSerializer.Deserialize<FwResult>(respContentString);
                                _logger.LogInformation("推送通话报告结果：" + respContentString);
                            }
                            catch (Exception ex)
                            {
                                //新增未推送成功记录
                                var callRecordFail = new CallRecordFail();
                                callRecordFail.FailType = EFailType.CallRecord;
                                callRecordFail.FailString = respContentString;
                                callRecordFail.FailReason = ex.Message;
                                await _callRecordFailRepository.AddAsync(callRecordFail, cancellationToken);
                               
                                _logger.LogInformation("推送通话报告失败：" + ex.Message);
                                //throw new UserFriendlyException(ex.Message);
                            }
                            #endregion

                            #region 推送超时未接记录报告
                            //准备报告
                            //查询是否有15秒未接
                            var CallDetails = call.CallDetails;

                             var ringList = CallDetails?.Where(x => x.CallStatus == ECallStatus.Ring).ToList();
                            //多个振铃，包括转接（需要验证转接分机）
                            if (ringList!=null && ringList.Count>1)
                            {
                                List<OutCallNotReceivedDto> NotReceivedList = new List<OutCallNotReceivedDto>();

                                int count = CallDetails?.Count(x => x.CallStatus == ECallStatus.Answer)??0;
                                if (count>0)
                                {
                                    ringList.RemoveRange(ringList.Count-2,count);
                                }
                                //验证是否被接过
                                for (int i = 0; i < ringList.Count; i++)
                                {
                                    
                                    //bool IsReceived = CallDetails?.Any(x => x.CallStatus == ECallStatus.Answer && x.AnswerNo == ringList[i].AnswerNo)??false;
                                    //未接听 加入到集合中
                                    //if (!IsReceived)
                                    //{
                                        var notReceivedEntity = new OutCallNotReceivedDto();
                                        notReceivedEntity.CallId = ringList[i].CallId;
                                        notReceivedEntity.Cpn = ringList[i].FromNo;
                                        notReceivedEntity.Cdpn = ringList[i].ToNo;
                                        notReceivedEntity.Answered = ringList[i].AnswerNo;
                                        notReceivedEntity.RingTime = ringList[i].CreationTime;
                                        //验证是否在签入状态振铃
                                        var iswork = await _workRepository.AnyAsync(x => x.TelNo == notReceivedEntity.Answered && !x.EndTime.HasValue);
                                        notReceivedEntity.IsWorkRing = iswork;
                                        NotReceivedList.Add(notReceivedEntity);
                                    //}
                                }
                                if (NotReceivedList.Count>0)
                                {
                                    //推送报告
                                    try
                                    {
                                        using var client = _httpClientFactory.CreateClient();
                                        client.DefaultRequestHeaders.ConnectionClose = true;
                                        var requestContent = JsonSerializer.Serialize(NotReceivedList);
                                        _logger.LogInformation(requestContent);
                                        var content = new StringContent(requestContent, Encoding.UTF8, "application/json");
                                        var responseMessage = await client.PostAsync(_sendCallRecordOptions.Value.NotReceivedUrl, content, cancellationToken);
                                        var respContent = responseMessage.Content;
                                        respContentString = await respContent.ReadAsStringAsync(cancellationToken);
                                        var result = JsonSerializer.Deserialize<FwResult>(respContentString);
                                        _logger.LogInformation("推送未接听报告结果：" + respContentString);
                                    }
                                    catch (Exception ex)
                                    {
                                        //新增未推送成功记录
                                        var callRecordFail = new CallRecordFail();
                                        callRecordFail.FailType = EFailType.NotReceived;
                                        callRecordFail.FailString = respContentString;
                                        callRecordFail.FailReason = ex.Message;
                                        await _callRecordFailRepository.AddAsync(callRecordFail, cancellationToken);
                                        _logger.LogInformation("推送未接听报告失败：" + ex.Message);
                                        //throw new UserFriendlyException(ex.Message);
                                    }
                                }

                            }
                            //////////如果只有一条振铃记录或没有 就不推送

                            #endregion



                        }
                    }
                }
            }

           
        }
    }

    public class FwResult
    {
        public int code { get; set; }
        public string msg { get; set; }
    }
}
